@@ -47,4 +47,19 @@ free_header(header *h);
 inline void
 free_request(request *req);
 
+inline void
+dealloc_request(request *req);
+
+inline void
+request_list_fill(void);
+
+inline void
+request_list_clear(void);
+
+inline void
+header_list_fill(void);
+
+inline void
+header_list_clear(void);
+
 #endif
