@@ -193,34 +193,40 @@ writev_bucket(write_bucket *data)
     return 1;
 }
 
+static inline int 
+get_len(PyObject *v)
+{
+	Py_ssize_t res;
+	res = PyObject_Size(v);
+	if (res < 0 && PyErr_Occurred()){
+		PyErr_Clear();
+        return 0;
+    }
+	return (int)res;
+}
+
 static inline void
-auto_set_content_length(client_t *client, write_bucket *bucket, char *data, size_t datalen )
+set_content_length(client_t *client, write_bucket *bucket, char *data, size_t datalen )
 {
     PyObject *header, *length;
     char *value;
     Py_ssize_t valuelen;
 
     if(client->headers && !client->content_length_set){
-        if (PySequence_Check(client->response)) {
-            if (PySequence_Size(client->response) == 1) {
-                client->content_length_set = 1;
+        if (get_len(client->response) == 1) {
+            client->content_length_set = 1;
 #ifdef DEBUG
-                printf("set content_length %d \n", datalen);
+            printf("set content_length %d \n", datalen);
 #endif
-                length = PyString_FromFormat("%zu", datalen);
+            length = PyString_FromFormat("%zu", datalen);
 
-                header = Py_BuildValue("(sO)", "Content-Length", length);
-                Py_DECREF(length);
+            header = Py_BuildValue("(sO)", "Content-Length", length);
+            Py_DECREF(length);
 
-                PyList_Append(client->headers, header);
-                Py_DECREF(header); 
-                PyString_AsStringAndSize(length, &value, &valuelen);
-                add_header(bucket, "Content-Length", 14, value, valuelen);
-            }
-
-            if (PyErr_Occurred()){
-                PyErr_Clear();
-            }
+            PyList_Append(client->headers, header);
+            Py_DECREF(header); 
+            PyString_AsStringAndSize(length, &value, &valuelen);
+            add_header(bucket, "Content-Length", 14, value, valuelen);
         }
     }
 }
@@ -348,10 +354,10 @@ write_headers(client_t *client, char *data, size_t datalen)
     //header done 
     
     // check content_length_set
-    // auto set content_length
-    if(data){
-        auto_set_content_length(client, bucket, data, datalen); 
-    }
+    // set content_length
+    //if(data){
+    //    set_content_length(client, bucket, data, datalen); 
+    //}
 
     set2bucket(bucket, CRLF, 2);
     if(data){
@@ -634,10 +640,10 @@ inline int
 response_start(client_t *client)
 {
     int ret ;
-    enable_cork(client);
     if(client->status_code == 304){
         return write_headers(client, NULL, 0);
     }
+    enable_cork(client);
     if (CheckFileWrapper(client->response)) {
 #ifdef DEBUG
         printf("use sendfile \n");
